@@ -258,6 +258,9 @@ Puppet::Face.define(:infrastructure, '1.0.0') do
           orchestrator_database_name          => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['orchestrator_database_name'])},
           inventory_database_name             => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['inventory_database_name'])},
           host_action_collector_database_name => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['host_action_collector_database_name'])},
+          patching_database_name              => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['patching_database_name'])},
+          infra_assistant_database_name       => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['infra_assistant_database_name'])},
+          workflow_database_name              => #{PuppetX::Util::Apply.quoted_or_undef(puppet_enterprise['workflow_database_name'])},
           notify                              => Service['pe-puppetserver'],
         }
         EOS
